package com.mh.common.core.redis;

import lombok.extern.slf4j.Slf4j;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.data.redis.core.script.RedisScript;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.concurrent.TimeUnit;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 锁
 * @date 2025-06-06 16:08:13
 */
@Slf4j
@Component
public class RedisLock {

    private final StringRedisTemplate redisTemplate;

    public RedisLock(StringRedisTemplate redisTemplate) {
        this.redisTemplate = redisTemplate;
    }

    /**
     * 获取锁
     */
    public boolean lock(String key, String requestId, long expireTimeInSeconds) {
        Boolean success = redisTemplate.opsForValue().setIfAbsent(key, requestId, expireTimeInSeconds, TimeUnit.SECONDS);
        return Boolean.TRUE.equals(success);
    }

    /**
     * 尝试获取锁（带超时）
     */
    public boolean tryLock(String key, String requestId, long expireTime, long timeoutMs) throws InterruptedException {
        long startTime = System.currentTimeMillis();
        while (System.currentTimeMillis() - startTime < timeoutMs) {
            if (lock(key, requestId, expireTime)) {
                return true;
            }
            Thread.sleep(50);
        }
        return false;
    }

    /**
     * 释放锁（使用 Lua 脚本保证原子性）
     */
    public void unlock(String key, String requestId) {
        String script = "if redis.call('get', KEYS[1]) == ARGV[1] then return redis.call('del', KEYS[1]) else return 0 end";
        RedisScript<Long> redisScript = RedisScript.of(script, Long.class);

        Long result = redisTemplate.execute(redisScript, Collections.singletonList(key), requestId);

        if (result == null || result == 0) {
            log.warn("释放锁失败，可能已被其他线程释放 key={}", key);
        }
    }
}
