package com.mh.framework.netty;

import com.mh.common.core.domain.AjaxResult;
import com.mh.common.core.domain.entity.CollectionParamsManage;
import com.mh.common.core.domain.entity.GatewayManage;
import com.mh.common.core.domain.entity.OrderEntity;
import com.mh.common.core.redis.RedisCache;
import com.mh.common.core.redis.RedisLock;
import com.mh.common.utils.ModbusUtils;
import com.mh.common.utils.NettyTools;
import com.mh.common.utils.StringUtils;
import com.mh.framework.netty.session.ServerSession;
import com.mh.framework.netty.session.SessionMap;
import com.mh.system.mapper.device.CollectionParamsManageMapper;
import com.mh.system.mapper.device.GatewayManageMapper;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.UUID;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.TimeUnit;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description netty实现类
 * @date 2025-06-06 15:13:23
 */
@Slf4j
@Service
public class NettyServiceImpl implements INettyService {

    @Resource
    private CollectionParamsManageMapper collectionParamsManageMapper;

    @Resource
    private GatewayManageMapper gatewayManageMapper;

    @Resource
    private RedisCache redisCache;

    @Resource
    private RedisLock redisLock;

    @Override
    public boolean sendOrder(List<OrderEntity> changeValues) {
        for (OrderEntity changeValue : changeValues) {
            String cpmId = changeValue.getId();
            CollectionParamsManage collectionParamsManage = collectionParamsManageMapper.selectById(cpmId);
            if (null == collectionParamsManage) {
                return false;
            }
            GatewayManage gatewayManage = gatewayManageMapper.selectById(collectionParamsManage.getGatewayId());
            if (null == gatewayManage || StringUtils.isEmpty(gatewayManage.getHeartBeat())) {
                return false;
            }
            ConcurrentHashMap<String, ServerSession> map = SessionMap.inst().getMap();
            Set<Map.Entry<String, ServerSession>> entries = map.entrySet();
            boolean flag = false;
            String keyVal = null;
            for (Map.Entry<String, ServerSession> entry : entries) {
                String key = entry.getKey();
                if (key.contains(gatewayManage.getHeartBeat())){
                    flag = true;
                    keyVal = key;
                    break;
                }
            }
            if (flag) {
                ServerSession serverSession = map.get(keyVal);
                // 目前只有DTU，modbus方式，只创建modbus先
                String controlCode = ModbusUtils.createControlCode(collectionParamsManage.getMtCode(),
                        changeValue.getType(),
                        collectionParamsManage.getRegisterAddr(),
                        changeValue.getParam());
                if (StringUtils.isEmpty(controlCode)) {
                    log.error("创建控制码失败");
                    return false;
                }

                String requestId = UUID.randomUUID().toString(); // 唯一标识当前请求
                String lockKey = "lock:order_send:" + gatewayManage.getHeartBeat(); // 按网关分锁

                try {
                    if (!redisLock.tryLock(lockKey, requestId, 10, 10)) {
                        log.warn("获取锁失败，当前操作繁忙");
                        return false;
                    }
                    // 初始化发送指令
                    NettyTools.initReceiveMsg("order_wait");
                    // 设置缓存，方便在netty中判断发送的指令
                    redisCache.setCacheObject("order_send", controlCode, 10, TimeUnit.SECONDS);
                    // 发送指令
                    serverSession.getChannel().writeAndFlush(ModbusUtils.createByteBuf(controlCode));
                    // 等待指令
                    if (NettyTools.waitReceiveMsg("order_wait")) {
                        log.error("发送指令成功,心跳包:{}", gatewayManage.getHeartBeat());
                        return true;
                    } else {
                        log.error("发送指令异常,心跳包:{}", gatewayManage.getHeartBeat());
                        return false;
                    }
                } catch (InterruptedException e) {
                    log.error("发送指令异常", e);
                } finally {
                    redisLock.unlock(lockKey, requestId);
                }
            }
            log.error("当前设备不在线,心跳包:{}",gatewayManage.getHeartBeat());
            return false;
        }
        return false;
    }
}
