package com.mh.system.service.policy.impl;

import com.mh.common.core.domain.dto.DeviceMonitorDTO;
import com.mh.common.core.domain.entity.CollectionParamsManage;
import com.mh.common.core.domain.entity.PolicyManage;
import com.mh.common.utils.StringUtils;
import com.mh.system.mapper.device.CollectionParamsManageMapper;
import com.mh.system.mapper.policy.PolicyManageMapper;
import com.mh.system.service.policy.IPolicyManageService;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.util.*;
import java.util.stream.Collectors;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 策略管理服务实现类
 * @date 2025-04-01 15:05:43
 */
@Service
public class PolicyManageServiceImpl implements IPolicyManageService {

    private final PolicyManageMapper policyManageMapper;

    private final CollectionParamsManageMapper collectionParamsManageMapper;

    public PolicyManageServiceImpl(PolicyManageMapper policyManageMapper, CollectionParamsManageMapper collectionParamsManageMapper) {
        this.policyManageMapper = policyManageMapper;
        this.collectionParamsManageMapper = collectionParamsManageMapper;
    }

    @Override
    public List<PolicyManage> selectListByCpmIds(List<CollectionParamsManage> collectionParamsManages) {
        if (collectionParamsManages.isEmpty()) {
            return List.of();
        }
        List<PolicyManage> resultList = new ArrayList<>();
        for (CollectionParamsManage collectionParamsManage : collectionParamsManages) {
            if (StringUtils.isEmpty(collectionParamsManage.getId())) {
                throw new RuntimeException("未找到该时间策略值");
            }
            PolicyManage policyManage = policyManageMapper.selectOntByCpmId(collectionParamsManage.getId());
            if (null == policyManage) {
                throw new RuntimeException("未找到该时间策略值");
            }
            resultList.add(policyManage);
        }
        return resultList;
    }

    @Override
    public int updateTimeValue(String policyId, String timeValue) {
        String cpmId = policyManageMapper.selectCpmId(policyId);
        if (StringUtils.isEmpty(cpmId)) {
            throw new RuntimeException("未找到该时间策略值");
        }
        return collectionParamsManageMapper.updateCurValueById(cpmId, new BigDecimal(timeValue), new Date());
    }

    @Override
    public List<?> selectDDCTimeList(String systemType, String funPolicyType, String houseId) {
        List<PolicyManage> policyManages = policyManageMapper.selectPolicyListByParams(systemType, funPolicyType, houseId);
        Map<String, List<PolicyManage>> listMap = policyManages.stream()
                // 先排序（例如按 policyType）
                .sorted(Comparator.comparing(PolicyManage::getPolicyType))
                // 处理每个PolicyManage对象的curValue字段
                .peek(policy -> {
                    policy.setCurValue(BigDecimal.valueOf(policy.getCurValue().intValue())); // 除以1000并保留整数
                })
                // 分组时使用 LinkedHashMap 来保持插入顺序
                .collect(Collectors.groupingBy(
                        PolicyManage::getPolicyName,  // 分组键
                        LinkedHashMap::new,           // 使用 LinkedHashMap 保持顺序 ✅
                        Collectors.toList()           // 下游收集器
                ));

        // 转换为 DeviceMonitorDTO 列表
        List<DeviceMonitorDTO> result = new ArrayList<>();
        listMap.forEach((name, values) -> {
            DeviceMonitorDTO dto = new DeviceMonitorDTO();
            dto.setName(name);
            dto.setValues(values);
            result.add(dto);
        });

        return result;
    }

    @Override
    public List<?> selectPolicyList(String systemType, String funPolicyType) {
//        QueryWrapper<PolicyManage> queryWrapper = new QueryWrapper<>();
//        queryWrapper.eq("system_type", systemType);
//        queryWrapper.eq("fun_policy_type", funPolicyType);
//        queryWrapper.orderByAsc("order_num");
        List<PolicyManage> policyManages = policyManageMapper.selectPolicyList(systemType, funPolicyType);
        // policyManages使用stream流，根据policyType进行数据分组，
        Map<String, List<PolicyManage>> listMap = policyManages.stream()
                // 先按PolicyType排序
                .sorted(Comparator.comparing(PolicyManage::getPolicyType))
                // 处理每个PolicyManage对象的curValue字段
                .peek(policy -> {
                    // 自动开关机时间，反馈是ms，页面是s，所以除以1000
//                    if (funPolicyType.equals("2")) {
                    policy.setCurValue(BigDecimal.valueOf((policy.getCurValue() == null ? new BigDecimal("0") : policy.getCurValue())
                            .divide(new BigDecimal((int) Math.pow(10, policy.getDigits()))).intValue())); // 除以1000并保留整数
//                    }
                    // 判断系统启动模式值是多少，如果是1：一键启动，如果是2：定时启动
                    if (policy.getPolicyType().equals("5") && policy.getFunPolicyType().equals("1")) {
                        // 查询当前系统启动模式值
                        CollectionParamsManage collectionParamsManage = collectionParamsManageMapper.selectById(policy.getCpmId());
                        if (null != collectionParamsManage) {
                            int systemStartMode = collectionParamsManageMapper.selectById(policy.getCpmId()).getCurValue().intValue();
                            // 匹配orderNum值，如果相等，哪个的curValue=1
                            if (policy.getOrderNum() == systemStartMode) {
                                policy.setCurValue(BigDecimal.valueOf(1));
                            } else {
                                policy.setCurValue(BigDecimal.valueOf(0));
                            }
                        } else {
                            policy.setCurValue(BigDecimal.valueOf(0));
                        }
                    }
                })
                // 分组时使用 LinkedHashMap 来保持插入顺序
                .collect(Collectors.groupingBy(
                        PolicyManage::getPolicyName,  // 分组键
                        LinkedHashMap::new,           // 使用 LinkedHashMap 保持顺序 ✅
                        Collectors.toList()           // 下游收集器
                ));
        // 在遍历赋值给DeviceMonitorDTO，key给name，value给list
        List<DeviceMonitorDTO> list = new ArrayList<>();
        listMap.keySet().forEach(k -> {
            DeviceMonitorDTO deviceMonitorDTO = new DeviceMonitorDTO();
            deviceMonitorDTO.setName(k);
            deviceMonitorDTO.setValues(listMap.get(k));
            list.add(deviceMonitorDTO);
        });
        return list;
    }
}
