package com.mh.quartz.util;

/**
 * @author LJF
 * @version 1.0
 * @project EEMCS
 * @description 模糊PID控制算法
 * @date 2025-06-04 09:47:16
 */
public class FuzzyPIDControlUtil {
    // PID参数
    private final double kp;   // 比例增益
    private final double ki;   // 积分增益
    private final double kd;   // 微分增益

    // 控制器状态
    private double prevError;
    private double integral;
    private double derivative;

    // 模糊规则参数
    private final double[] errorLevels = {-6, -3, -1, 0, 1, 3, 6};  // 温度误差级别(℃)
    private final double[] dErrorLevels = {-3, -1, 0, 1, 3};         // 误差变化率级别(℃/min)
    private final double[] kpAdjust = {1.5, 2.0, 2.5, 3.0, 4.0};    // Kp调整因子 (增强)
    private final double[] kiAdjust = {0.3, 0.7, 1.0, 1.3, 1.7};    // Ki调整因子

    // 阀门限制
    private static final double MIN_VALVE = 0.0;    // 最小开度(0%)
    private static final double MAX_VALVE = 100.0;   // 最大开度(100%)

    public FuzzyPIDControlUtil(String kp, String ki, String kd) {
        this.kp = Double.parseDouble(kp);
        this.ki = Double.parseDouble(ki);
        this.kd = Double.parseDouble(kd);
        this.prevError = 0;
        this.integral = 0;
        this.derivative = 0;
    }

    // 模糊推理计算PID参数调整因子
    private double[] fuzzyInference(double error, double dError) {
        // 模糊化：计算误差和误差变化率的隶属度
        double[] errorMembership = calculateMembership(error, errorLevels);
        double[] dErrorMembership = calculateMembership(dError, dErrorLevels);

        // 模糊规则库
        double kpAdjustSum = 0.0;
        double kiAdjustSum = 0.0;
        double weightSum = 0.0;

        // 应用模糊规则 (增强大误差时的响应)
        for (int i = 0; i < errorMembership.length; i++) {
            for (int j = 0; j < dErrorMembership.length; j++) {
                double weight = errorMembership[i] * dErrorMembership[j];
                if (weight > 0) {
                    // 增强大误差时的响应
                    int kpIndex;
                    if (Math.abs(error) > 3) { // 大误差
                        kpIndex = Math.min(Math.max(i + j, 0), kpAdjust.length - 1);
                    } else {
                        kpIndex = Math.min(Math.max(i + j - 1, 0), kpAdjust.length - 1);
                    }

                    // Ki调整：小误差时增强积分作用
                    int kiIndex;
                    if (Math.abs(error) < 1) { // 小误差
                        kiIndex = Math.min(Math.max(3 + j, 0), kiAdjust.length - 1);
                    } else {
                        kiIndex = Math.min(Math.max(2 + j, 0), kiAdjust.length - 1);
                    }

                    kpAdjustSum += weight * kpAdjust[kpIndex];
                    kiAdjustSum += weight * kiAdjust[kiIndex];
                    weightSum += weight;
                }
            }
        }

        // 反模糊化 (加权平均)
        double kpFactor = weightSum > 0 ? kpAdjustSum / weightSum : 1.0;
        double kiFactor = weightSum > 0 ? kiAdjustSum / weightSum : 1.0;

        return new double[]{kpFactor, kiFactor, 1.0}; // Kd不调整
    }

    // 计算隶属度 (三角隶属函数)
    private double[] calculateMembership(double value, double[] levels) {
        double[] membership = new double[levels.length];

        for (int i = 0; i < levels.length; i++) {
            if (i == 0) {
                membership[i] = (value <= levels[i]) ? 1.0 :
                        (value < levels[i+1]) ? (levels[i+1] - value) / (levels[i+1] - levels[i]) : 0.0;
            } else if (i == levels.length - 1) {
                membership[i] = (value >= levels[i]) ? 1.0 :
                        (value > levels[i-1]) ? (value - levels[i-1]) / (levels[i] - levels[i-1]) : 0.0;
            } else {
                if (value >= levels[i-1] && value <= levels[i]) {
                    membership[i] = (value - levels[i-1]) / (levels[i] - levels[i-1]);
                } else if (value >= levels[i] && value <= levels[i+1]) {
                    membership[i] = (levels[i+1] - value) / (levels[i+1] - levels[i]);
                } else {
                    membership[i] = 0.0;
                }
            }
        }

        return membership;
    }

    // 计算控制输出 (阀门开度) - 修复了符号问题
    public double calculate(double setpoint, double currentValue, double dt) {
        // 计算误差项 - 修复：当前值高于设定值需要冷却，误差应为正
        double error = currentValue - setpoint;

        // 计算微分项 (基于误差变化率)
        if (dt > 0) {
            derivative = (error - prevError) / dt;
        }

        // 模糊调整PID参数
        double[] adjustments = fuzzyInference(error, derivative);
        double adjKp = kp * adjustments[0];
        double adjKi = ki * adjustments[1];
        double adjKd = kd * adjustments[2];

        // 计算积分项 (带抗饱和)
        integral += error * dt;

        // 抗饱和限制
        double maxIntegral = MAX_VALVE / (adjKi + 1e-5);
        if (Math.abs(integral) > maxIntegral) {
            integral = Math.signum(integral) * maxIntegral;
        }

        // PID计算 - 修复：误差为正时需要正输出打开阀门
        double output = adjKp * error + adjKi * integral + adjKd * derivative;

        // 保存误差用于下次计算
        prevError = error;

        // 阀门开度限制
        return Math.min(Math.max(output, MIN_VALVE), MAX_VALVE);
    }

    // 重置控制器状态
    public void reset() {
        integral = 0;
        prevError = 0;
        derivative = 0;
    }
}
